<!DOCTYPE HTML>

<html>
  <head>
    <title>Simple Benchmark</title>
  </head>
  <body>
    <h1><%= header %></h1>
    <% unless item.empty? %>
      <ul>
      <% for i in item %>
        <% if i[:current] %>
          <li><strong><%= i[:name] %></strong></li>
        <% else %>
          <li><a href="<%= i[:url] %>"><%= i[:name] %></a></li>
        <% end %>
      <% end %>
      </ul>
    <% else %>
      <p>The list is empty.</p>
    <% end %>
  </body>
</html>